#ifndef _USER_H_
#define _USER_H_

#include "lasers_robots.h"


/****
     FUNCTION: Global_localization
     FROM :    Map_mgr.c
  Compute  a list of valid hypothesis
  about the robot location when it acquired the data
  stored in the image n_data. This location 
  is computed according to the current map stored
  in Map_ind (index) and Map (structure itself) (see Map_mgr.c).

  No assumption is made about the location of the robot.
  However, the user can restrict the search to a particular
  angle area by using Set_angle_restriction() described below.
  This restriction will automatically be released at the end of
  this function.

  The option "op" must be set to 1 if you want to filter wrong 
  locations as explained in the manual.

  Fills the array possloc with all the possible locations and
  returns the index on the best one in this array.
*****/

long  Global_localization(long n_data,long *possloc, long op, int maxarraysize);


/****
     FUNCTION: Local_localization
     FROM :    Map_mgr.c
  Compute  a list of valid hypothesis
  about the robot location when it acquired the data
  stored in the image n_data. This location 
  is computed according to the current map stored
  in Map_ind (index) and Map (structure itself) (see Map_mgr.c).

  This function takes the location xl,yl as an estimate of
  the robot position. In order to work, the aspect ratio from 
  the approximate location must contain at least 2 non-parallel
  segments visible from the actual position of the robot.
  
  In addition, the user can restrict the search to a particular
  angle area by using Set_angle_restriction() described below.
  This restriction will automatically be released at the end of
  this function.

  The option "op" must be set to 1 if you want to filter wrong 
  locations as explained in the manual.

  Fills the array possloc with all the possible locations and
  returns the index on the best one in this array.

  Defined in Map_mgr.c
*****/


long  Local_localization(long n_data,double xl,double yl,long *possloc, long op, int maxarraysize, laserConfiguration *sick_config);



/****************
     FUNCTION: FilterBadLocations
     FROM :    disambiguate.c
  Filters bad hypothesis using the data stored in im
  and the robot encoder position corresponding to
  the time of the data acquisition:
  

  The filter  uses 2 ways:

  -it tests if the current location array 
   computed from  the transform corresponding
   to each hypothesis is inside the limit of the boundary (if any) 
   the user defined while entering the map.

  -if the location satisfies the boudary constraint, then computes
   the maximum distance of a point in 'im' behind a non-transparent object
   belonging to the map. If this distance is higher than 'maxdist', the
   hypothesis is rejected. Because of the drift of the robot, one wants to
   set this distance :
    -short (4-5 inches, even less) , right after the localization is done for a more efficient
     filtering
    -long (10-20 inches, maybe more...) as the robot moves along.

  Fills the array possloc with all the remaining possible transformations 

  Then, the function frees the structure containing the new data
  and returns the index on the best remaining location.

  Returns -1 if no remaining possibilities

  Defined in disambiguate.c
***************/

long FilterBadLocations(long im, long *possloc,double maxdist, int maxarraysize);


/***************
     FUNCTION: LasSegmentation
     FROM :    disambiguate.c
  takes a number of points and an array of pixel
  values;
  returns an index on a segment image.
  The segments are computed according to
  to setting made by conf_sg_host().
  (the same as conf_sg but on the host side)

  
******************/

long LasSegmentation(long,long *);


/*************
     FUNCTION: LasScanSegmentation
     FROM :    image_mgr.c
 Creates a global image of laser points
 from an array of laser_val_array[3] snap shots.
 taken from the position
 x = laser_val_array[0], y = laser_val_array[1], th = laser_val_array[2].
 laser_val_array[4] = turret angle of the first snap shot
 laser_val_array[5] = number of point of the first snap shot
 then come the first snap shot pixel values
 etc ....
*************/

long  LasScanSegmentation(long *);



/*******
     FUNCTION: Init_map
     FROM :    Map_mgr.c
		     

  Function to create a map;

  Initialize the internal map structure
  Must be called each time one
  want to create a new map
*******/

void Init_map(void);


/******
     FUNCTION: Init_boundary
     FROM :    Map_mgr.c

  Initialize the internal boundary structure.
  See ReadMap for a complete example.
*******/
void Init_boundary(void);

/***************
     FUNCTION: NewSegment
     FROM :    Map_mgr.c

  adds a segment to the map;
  Segments are understood in the server map 
  format: the free space is on the right side
  when walking from x1,y1 to x2,y2 and the unit is tenth of inches.
  
  
  See ReadMap() below for an example.

  
****************/
void NewSegment(double x1,double y1,double x2,double y2);

/*********
     FUNCTION: NewSegmentBound
     FROM :    Map_mgr.c
  adding a segment to the boundary
*********/

void NewSegmentBound(double x1,double y1,double x2,double y2);

/*********
     FUNCTION: CloseMap
     FROM :    Map_mgr.c
*********/

void CloseMap(void);


/*****************
     FUNCTION: ReadMap
     FROM :    Map_mgr.c

  Reads a file of segments in the server format (tenth of inches and
  free space on the right) describing:
  
  - a map: set of segments connected or not describing the 2d shape
  of the robot environment. The described objects are assumed to
  be STATIC (walls or objects which don't move). They also have to not
  be see-through. This is important. That doesn't mean your environment
  has to be static. The process will take unexpected occlusions into account.

  -a boundary (optional)
  
  This is different form the map in the sens
  that it is not used for generating hypothesis
  but rather to restrict the area of the possible locations.
  Consequently, the boundary can contain "virtual" walls
  which could not be visible. It is a good 
  way to define doors or windows which could be open or
  closed but delimiting an area outside of which the robot cannot be.
  (outside doors and windows of a building, for example).

  The boundary can contain segments from the map. It is assumed
  to be a closed polygon. You don't need to repeat the possible segments
  of the map into the boundary. It needs only the extra segments necessary 
  to close the polygon. Consequently, a good way to define map + boundary is:

  - enter the entire map using NewSegment() described below.

  - enter all the extra segments for delimiting the boundary using
    NewSegmentBound() described below.

  - call CloseMap() for computing the actual closed polygon.

  NOTE: if CloseMap() can't find a closed polygon, the boundary
  information will be igored. If there is no extra closing segments,
  the function CloseMap() will try to find an external closed
  polygon out of the segments of the map. Thus, a closed map will
  automatically define a boundary.

  You don't need a boundary to perform the localization, 
  neither the disambiguating. However, without boundary, the localization
  process will generate more possible locations and the disambiguating
  will take a noticeable extra time to perform its job.

  
  The format of the file is assumed to be:

  - an integer for the number of the map segments
  - a list of segments (in double) x1,y1,x2,y2.
  - an integer for the number of extra closing segments (if any)
  - a list of segments (in double) x1,y1,x2,y2.
******************/
void ReadMap(char *);



/***************
 * FUNCTION:     RetrieveMap
 * FROM:         Map_builder.c
 * PURPOSE:      retreives the map stored in 
                 *file in internal map builder format.
		 The map had to be saved by write_file_sg()
		 (see doc localization).
		 The local static variables Map and Map_ind
		 are set so one can continue building up
		 this same map => allows incremental map
		 building.
		 This function will also make sure localization
		 software and map building will share the same map,
		 which is good.

 * ARGUMENTS:    char *file :
 * ALGORITHM:    
 * RETURN:       long 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/
    


long RetrieveMap(const char *file);

/*******
 * FUNCTION:     Set_loc_angle
 * FROM:         match.c
 * PURPOSE:      setting parameters
*******/


void Set_loc_angle(double,double);

/*******
 * FUNCTION:     Set_map_unaccuracy
 * FROM:         Map_mgr.c
 * PURPOSE:      
*******/

void Set_map_unaccuracy(double);


/***********************
 * FUNCTION:     SetMinLength
 * FROM:         match.c 
 * PURPOSE: Data segments could be very inaccurate when 
too small. This function allows you to 
reject such small segments by setting the 
minimum length for a data segment to be used 
by the localization process
When you are sure the robot sees long segments 
a high value may speed up the computation
However, a small value (even 0) should be used
in case only little (or broken) segments are available
The default value is 4 inches.
************************/

void SetMinLength(double);



/********
 * FUNCTION:     write_file_sg, read_file_sg
 * FROM:         image_mgr.c 
 * PURPOSE:   reading/writing data files
*********/

long write_file_sg(long im);
long read_file_sg(const char *file);



/*******************
 * FUNCTION:     read_char freeze_on_rchar
 * FROM:         las_io.c
 * PURPOSE:    special function for
  keyboard control:

  both read a character (only one) without
  waiting for a carriage return and put the 
  character in the pointer c.

  -read_char doesn't wait at all and return
  1 if there is a key pressed, 0 if not.

  -freeze_on_rchar wait indefinitely untill
  one character is entered.


************************/

long read_char(char *c);

long freeze_on_rchar(char *c);


/****************
 * FUNCTION:     Copy_image_segment
 * FROM:         image_mgr.c
 * PURPOSE:   copies an internal image segment in the array
  laser_segment  (must be allocated).
  Useful for interfacing the result of a segmentation 
  (see LasSegmentation above).

  laser_segment[0] = number 0f recorded segments;

  max : max size of laser_segment

  x1,y1,x2,y2;

  unit: tenth of inch

  orientation : free-space on the right side

********************/

long Copy_image_segment(long im,long *laser_segment, int maxarraysize);




/****************
 * FUNCTION:     DataIntegration
 * FROM:         Map_builder.c
 * PURPOSE:     returns an array of segments composing the new map.
  the first 4 integers: x,y,steering,turret corrected
  then:
  number of segments (array[4])
  x1,y1,x2,y2

  Units: tenth of inch, tenth of degree.

  WARNING: this array is allocated inside the function.
           you're responsible of freeing it.
*****/

long *DataIntegration(long data, laserConfiguration *sick_config);


/****
 * FUNCTION:     Gfree_image
 * FROM:         image_mgr.c
 * PURPOSE:   Freeing the internal structure pointed to by the ID n
*****/

void Gfree_image(long n);


/***************
 * FUNCTION:     RemoveSegment
 * FROM:         Map_builder.c
 * PURPOSE:      removes the map segment matching the
                 coordinates (x1,y1) and (x2,y2).
		 Coordinates are expressed in tenth of inch,
		 free space on the right going from (x1y1) to
		 (x2, y2).
 * ARGUMENTS:    long x1 :
 *               long y1 :
 *               long x2 :
 *               long y2 :
 * ALGORITHM:    
 * RETURN:       1 if the segment has been found and removed, 0 otherwise
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

int RemoveSegment(double x1,double y1,double x2,double y2);


/***************
 * FUNCTION:     GetCurrentMap
 * FROM:         Map_builder.c
 * PURPOSE:        returns an array of segments composing the current map.
                   the first 4 integers: x,y,steering,turret corrected
		   then:
		   number of segments (array[4])
		   for i=1 t0  array[4]
		   x1[i] = array[4*i + 1]
		   y1[i] = array[4*i + 2]
		   x2[i] = array[4*i + 3]
		   y2[i] = array[4*i + 4]
		   end for
		   
		   
		   Units: millimeter, milliradiants.
		   
		   Orientation: free space on right side when going from
		   (x1,y1) to (x2,y2).
		   
		   WARNING: this array is allocated inside the function.
		   you're responsible for freeing it.
 * ARGUMENTS:    None
 * ALGORITHM:    
 * RETURN:       long *
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

long *GetCurrentMap(void);


/***************
 * FUNCTION:     GetMapIndex
 * FROM:         Map_builder.c
 * PURPOSE:      returns the index on the current map image
                
 * ARGUMENTS:    None
 * ALGORITHM:    
 * RETURN:       int 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

long GetMapIndex(void);


/***************
 * FUNCTION:     FilterOutData
 * FROM:         Map_builder.c
 * PURPOSE:      helper for filtering
 * ARGUMENTS:    
 * ALGORITHM:    
 * RETURN:       
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

void FilterOutData(int image);

/***************
 * FUNCTION:     ReadSession, SaveSession, OpenSaveSession
 * FROM:         session.c
 * PURPOSE:      
 * ARGUMENTS:    
 * ALGORITHM:    
 * RETURN:       
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

int ReadSession(long n);

int SaveSession (long n);

int OpenSaveSession(const char *name,long start);

/***************
 * FUNCTION:     ChangeMapCoordinates
 * FROM:         Map_builder.c
 * PURPOSE:      
 * ARGUMENTS:    
 * ALGORITHM:    
 * RETURN:       
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

long * ChangeMapCoordinates(double *start, double *end);


/***************
 * FUNCTION:     MergeSegments
 * FROM:         Map_builder.c
 * PURPOSE:      
 * ARGUMENTS:    
 * ALGORITHM:    
 * RETURN:       
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/
int MergeSegments(double *seg);

/***************
 * FUNCTION:     RotateMap
 * FROM:         Map_builder.c
 * PURPOSE:      
 * ARGUMENTS:    
 * ALGORITHM:    
 * RETURN:       
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/
long * RotateMap(double angle);

/***************
 * FUNCTION:     Erase_map
 * FROM:         Map_builder.c
 * PURPOSE:      
 * ARGUMENTS:    
 * ALGORITHM:    
 * RETURN:       
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/
void Erase_map(void);


/* ---------------------------------------------------------------------- */
/* FUNCTIONS  for server_skel.c ONLY */
/* ---------------------------------------------------------------------- */

/** 
    IntersectsSegments (from Map_builder.c and for server_skel.c)
**/
  
int IntersectsSegments(long x11,long y11,long x12,long y12, 
			long x21,long y21,long x22,long y22);
int IntersectsSegmentsD(double x11,double y11,double x12,double y12, 
			double x21,double y21,double x22,double y22);
/*** 
     MapTrData  (from Map_builder.c and for server_skel.c)

  in case we could not relocate, tries to match directly
  - (still relies on the aspect) 
  - for each data segment :
      -computes the probabilistic distance with all segments
       in the aspect. Takes the closest: if the distance is
       less than thres: 
         - computes a new position for the robot
         - adds the matching to a list of matching
      -appends data to map
***/

PDBL *MapTrData(igla *data,double *pos,double *vpos,double thres);



#endif /* _USER_H_ */
